@@ -1,20 +1,18 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
+from .yoc import *
+from .subcmds import all_commands
+import optparse
+import sys
+import os
 
 import warnings
 warnings.filterwarnings("ignore", category=SyntaxWarning)
 
-import os
-import sys
-import optparse
-
-from .subcmds import all_commands
-from .yoc import *
-
 
 global_options = optparse.OptionParser(
     usage="yoc COMMAND [ARGS]"
@@ -58,24 +56,24 @@ class YocCommand:
             cmd = self.commands[name]
 
         except KeyError:
-            print("yoc: '%s' is not a yoc command.  See 'yoc help'." %
-                  name, file=sys.stderr)
+            put_string("yoc: '%s' is not a yoc command.  See 'yoc help'." %
+                       name, file=sys.stderr)
             return 1
 
         try:
             copts, cargs = cmd.OptionParser.parse_args(argv)
             copts = cmd.ReadEnvironmentOptions(copts)
         except Exception as e:
-            print('error: in `%s`: %s' % (' '.join([name] + argv), str(e)),
-                  file=sys.stderr)
-            print('error: manifest missing or unreadable -- please run init',
-                  file=sys.stderr)
+            put_string('error: in `%s`: %s' % (' '.join([name] + argv), str(e)),
+                       file=sys.stderr)
+            put_string('error: manifest missing or unreadable -- please run init',
+                       file=sys.stderr)
             return 1
         try:
             cmd.ValidateOptions(copts, cargs)
             result = cmd.Execute(copts, cargs)
         except Exception as e:
-            print("YocCommand error:", e)
+            put_string("YocCommand error:", e)
             pass
         return result
 
