@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -11,6 +11,8 @@ import json
 import zipfile
 import shutil
 import threadpool
+import codecs
+import copy
 
 from .tools import *
 from .log import logger
@@ -23,6 +25,8 @@ class Component:
         self.conf = conf
         self.path = os.path.dirname(filename)
         self.name = os.path.basename(self.path)
+        self.tag = ''
+        self.license = ''
         self.type = ''
         self.version = ''
         self.depends = []
@@ -33,7 +37,7 @@ class Component:
         self.repo_url = ''
         self.repo = None
         self.yoc_version = ''
-        self.source_file = []
+        self.source_files = []
         self.defconfig = {}
         self.build_config = None
         self.hw_info = None
@@ -41,6 +45,7 @@ class Component:
         self.exports = []
 
         self.depends_on = []  # 该组件被哪个组件依赖
+        self.need_build = True
 
     def json_dumps(self):
         js = {
@@ -83,8 +88,8 @@ class Component:
             repo_url = os.path.join(self.conf.repo, '%s.git' % self.name)
             prj = GitRepo(self.path, repo_url)
 
-            # print("%s(%s), clone %s %s ..." % (self.name, self.version, self.repo_url, self.path))
-            # print("%s(%s) ..." % (self.name, self.version))
+            # put_string("%s(%s), clone %s %s ..." % (self.name, self.version, self.repo_url, self.path))
+            # put_string("%s(%s) ..." % (self.name, self.version))
             prj.pull(self.version, None)
 
         elif self.version in self.historyVersion.keys():
@@ -96,7 +101,6 @@ class Component:
             else:
                 zipf.extractall('.')
 
-
     def upload(self):
         repo_url = os.path.join(self.conf.repo, '%s.git' % self.name)
         try:
@@ -106,15 +110,14 @@ class Component:
             prj.import_path(self.path, self.version)
             # repo.branch_to_tag(self.name, 'v7.2-dev', 'V7.2.2')
         except:
-            print("component `%s` upload fial." % self.name)
+            put_string("component `%s` upload fial." % self.name)
 
         try:
             shutil.rmtree('/tmp/' + self.name)
         except Exception as e:
-            print(e)
+            put_string(e)
             exit(-1)
 
-
     def zip(self, path):
         zipName = os.path.join(
             path, '.cache', self.name + '-' + self.version + '.zip')
@@ -124,7 +127,7 @@ class Component:
 
         return zipName
 
-    def show(self, indent=0):
+    def show(self, indent=0, key=[]):
         if os.path.isdir(self.path):
             status = '*'
         else:
@@ -134,15 +137,15 @@ class Component:
         size = len(s1)
 
         text1, text2 = string_strip(self.description, 80)
-        print("%s%s %s %s - %s" %
-              (' '*indent, status, s1, ' ' * (40 - size), text1))
+        put_string("%s%s %s %s - %s" %
+                   (' '*indent, status, s1, ' ' * (40 - size), text1), key=key)
         while text2:
             text1, text2 = string_strip(text2, 80)
-            print(' ' * (46+indent) + text1)
+            put_string(' ' * (46 + indent) + text1, key=key)
 
     def info(self, indent=0):
-        for f in self.source_file:
-            print('%s%s' % (' '*indent, f))
+        for f in self.source_files:
+            put_string('%s%s' % (' '*indent, f))
 
     def load_package(self):
         if self.type:
@@ -160,14 +163,18 @@ class Component:
 
         self.name = pack.name
         self.type = pack.type
+        self.tag = pack.tag
+        self.license = pack.license
         self.description = pack.description
         self.yoc_version = pack.yoc_version
-        self.source_file = pack.source_file
-        self.build_config = pack.build_config
-        self.defconfig = pack.defconfig
-        self.installs = pack.install
-        self.exports = pack.export
-        self.hw_info = pack.hw_info
+        self.source_files = pack.source_file
+        self.build_config = copy.deepcopy(pack.build_config)
+        self.defconfig = copy.deepcopy(pack.defconfig)
+        self.installs = copy.deepcopy(pack.install)
+        self.exports = copy.deepcopy(pack.export)
+        self.hw_info = copy.deepcopy(pack.hw_info)
+        self.hw_info.update_path(self.path)
+        self.need_build = self.type not in ['board', 'chip']
 
         self.depends = []
         for d in pack.depends:
@@ -204,6 +211,19 @@ class Component:
                 libpaths.append(path)
         self.build_config.libpath = libpaths
 
+        # internal_include
+        internal_include = []
+        for var in self.build_config.internal_include:
+            var = varList.convert(var)
+            if var != None:
+                path = os.path.join(self.path, var)
+                if not (os.path.isdir(path) and os.path.exists(path)):
+                    logger.warning('%s is not exists or not directory.' % path)
+
+            if path not in internal_include:
+                internal_include.append(path)
+        self.build_config.internal_include = internal_include
+
         # libs
         libs = []
         for lib in self.build_config.libs:
@@ -212,42 +232,50 @@ class Component:
                 libs.append(lib)
         self.build_config.libs = libs
 
+        # depend:
+        depends = []
+        for dep in self.depends:
+            s = varList.convert(dep)
+            if s:
+                depends.append(s)
+        self.depends = depends
+
         # sources
         sources = []
-        for s in self.source_file:
+        for s in self.source_files:
             fn = varList.convert(s)
             if fn != None:
+                filename = os.path.join(self.path, fn)
+                if not os.path.isfile(filename):
+                    if not ('*' in filename or '?' in filename):
+                        logger.error('component `%s`: %s is not exists.' %
+                                     (self.name, filename))
+                        exit(-1)
                 sources.append(fn)
-        self.source_file = sources
+        self.source_files = sources
 
-        if self.hw_info.ld_script:
-            ld_script = varList.convert(self.hw_info.ld_script)
-            if ld_script:
-                self.hw_info.ld_script = os.path.join(self.path, ld_script)
+        def export_convert(convert):
+            exports = []
+            for ins in convert:
+                if ('source' not in ins) and ('dest' not in ins):
+                    continue
+                if not ins['source']:
+                    continue
+                srcs = []
+                for src in ins['source']:
+                    src = varList.convert(src)
+                    if src:
+                        srcs.append(src)
+                dest = varList.convert(ins['dest'])
+                if dest and src:
+                    exports.append({'dest': dest, 'source': srcs})
+            return exports
 
         # install
-        installs = []
-        for ins in self.installs:
-            srcs = []
-            for src in ins['source']:
-                srcs.append(varList.convert(src))
-            installs.append({
-                'dest': varList.convert(ins['dest']),
-                'source': srcs,
-            })
-        self.installs = installs
+        self.installs = export_convert(self.installs)
 
         # export
-        exports = []
-        for ins in self.exports:
-            srcs = []
-            for src in ins['source']:
-                srcs.append(varList.convert(src))
-            exports.append({
-                'dest': varList.convert(ins['dest']),
-                'source': srcs,
-            })
-        self.exports = exports
+        self.exports = export_convert(self.exports)
 
     def install(self, dest):
         for ins in self.installs:
@@ -260,7 +288,6 @@ class Component:
                 for s in glob.iglob(src):
                     fn = os.path.basename(s)
                     ds = os.path.join(path, fn)
-                    # print(ds, s)
                     shutil.copy2(s, ds)
 
     def export(self):
@@ -274,7 +301,6 @@ class Component:
                 for s in glob.iglob(src):
                     fn = os.path.basename(s)
                     ds = os.path.join(path, fn)
-                    # print(ds, s)
                     shutil.copy2(s, ds)
 
     def rename(self, new_name):
@@ -285,14 +311,14 @@ class Component:
         try:
             os.rename(old_path, new_path)
             filename = os.path.join(new_path, 'package.yaml')
-            with codecs.open(filename) as fh:
+            with codecs.open(filename, 'r', 'UTF-8') as fh:
                 lines = fh.readlines()
                 for i in range(len(lines)):
                     text = lines[i]
                     if text.find('name') >= 0:
                         lines[i] = text.replace(self.name, new_name)
                         break
-            with open(filename, 'w') as fh:
+            with codecs.open(filename, 'w', 'UTF-8') as fh:
                 fh.writelines(lines)
             self.path = new_path
             self.name = new_name
@@ -303,7 +329,7 @@ class Component:
 
             return True
         except Exception as e:
-            print(e)
+            put_string(e)
             return False
 
 
@@ -375,10 +401,10 @@ def string_strip(text, size):
     i = 0
     for c in text:
         if c >= '\u4E00' and c <= '\u9FA5':
-            # print(c)
+            # put_string(c)
             L += 2
         else:
-            # print('  ', c)
+            # put_string('  ', c)
             L += 1
         R += c
         i += 1
