@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 import os
@@ -125,9 +125,6 @@ class GitRepo:
             self.repo.git.push(
                 "--set-upstream", self.repo.remotes.origin, self.repo.head.ref)
 
-    def GetBranches(self):
-        return self.repo.branches
-
     def GetRemoteBranches(self):
         try:
             return self.repo.remote().refs
