@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 import sys
@@ -41,7 +41,7 @@ class OCC:
         global authorization, login_time
 
         if authorization and time.time() - login_time < 25 * 50:
-            return
+            return True
 
         cmd = '/api/base/auth/login'
         body = {
@@ -152,6 +152,9 @@ class OCC:
             response = connection.getresponse()
             if response.status == 200:
                 text = response.read()
+                if type(text) == bytes:
+                    text = bytes.decode(text)
+
                 js = json.loads(text)
 
                 if js['code'] != 0:
