@@ -1,12 +1,13 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
 import json
 import shutil
 import glob
+import codecs
 
 from .tools import *
 from .log import logger
@@ -17,19 +18,11 @@ desc_width = 50
 
 class HardwareInfo:
     def __init__(self, dic):
-        self.cpu_id = ''
-        self.vendor_name = ''
-        self.cpu_name = ''
-        self.arch_name = ''
-        self.chip_name = ''
-        self.board_name = ''
-        self.ld_script = ''
-        self.flash_program = ''
-
-        self.cpu_list = {}
+        self.reset()
 
         for k, v in dic.items():
-            if k in ['cpu_id', 'cpu_name', 'arch_name', 'chip_name', 'vendor_name', 'board_name', 'ld_script', 'flash_program'] and v:
+            if k in ['cpu_id', 'cpu_name', 'arch_name', 'chip_name', 'vendor_name', 'board_name',
+                'ld_script', 'flash_program', 'toolchain_prefix'] and v:
                 self.__dict__[k] = v
             else:
                 if type(v) == dict:
@@ -43,6 +36,32 @@ class HardwareInfo:
             text += k + ': ' + str(v)
         return text
 
+    def reset(self):
+        self.cpu_id = ''
+        self.vendor_name = ''
+        self.cpu_name = ''
+        self.arch_name = ''
+        self.chip_name = ''
+        self.board_name = ''
+        self.ld_script = ''
+        self.flash_program = ''
+        self.toolchain_prefix = ''
+
+        self.cpu_list = {}
+
+    def update_path(self, path):
+        if self.ld_script:
+            self.ld_script = os.path.join(path, self.ld_script)
+        cpu_list = {}
+        for k, v in self.cpu_list.items():
+            vv = {}
+            for k1, v1 in v.items():
+                if k1 == 'ld_script':
+                    v1 = os.path.join(path, v1)
+                vv[k1] = v1
+            cpu_list[k] = vv
+        self.cpu_list = cpu_list
+
     def merge(self, info, force=False):
         for k, v in self.__dict__.items():
             if force and info.__dict__[k]:
@@ -50,7 +69,8 @@ class HardwareInfo:
             elif not v and info.__dict__[k]:
                 self.__dict__[k] = info.__dict__[k]
 
-    def out(self):
+
+    def out(self, hw_key=None):
         text = ''
 
         if len(self.cpu_list) > 0:
@@ -60,15 +80,75 @@ class HardwareInfo:
                     for k, v in cpu.items():
                         text += '    {}: {}\n'.format(k, v)
         else:
-            for k in ['cpu_id', 'arch_name', 'vendor_name', 'chip_name', 'cpu_name', 'board_name', 'ld_script']:
+            for k in ['cpu_id', 'arch_name', 'vendor_name', 'chip_name', 'cpu_name', 'board_name', 'ld_script', 'toolchain_prefix']:
                 if self.__dict__[k]:
                     text += '  {}: {}\n'.format(k, self.__dict__[k])
 
         if self.flash_program:
             text += '  flash_program: {}\n'.format(self.flash_program)
 
+        text_head = '''
+## 第三部分：板级信息
+# hw_info:                                 # 对 solution、board、chip 三个字段的改进，统称为 hw_info 字段
+#   cpu_id: cpu0
+#   cpu_name: CK804EF
+#   vendor_name: t-head
+#   chip_name: chip_pangu                  # <可选项> 指定芯片组件名,未设置时，使用depends中 chip 第一个组件
+#   board_name: pangu_cpu0                 # <可选项> 指定开发板组件名，未设置时，使用depends中 board 第一个组件
+#   ld_script: configs/gcc_eflash.ld       # <可选项> 连接时使用 LD 脚本，当选把未设置时，使用对应的 board 的 LD 脚本
+#   flash_program: bin/flash_program.elf   # <可选项> 芯片全局烧写程序
+#   toolchain_prefix: csky-abiv2-elf       # <可选项> 编译器前缀
+#   cpu0:
+#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
+#     ld_script: configs/gcc_eflash.ld     # <必选项> 连接时使用 LD 脚本
+#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
+#     toolchain_prefix: csky-abiv2-elf     # <可选项> 编译器前缀
+#   cpu1:
+#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
+#     ld_script: configs/gcc_eflash.ld     # <可选项> 连接时使用 LD 脚本
+#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
+#     toolchain_prefix: csky-abiv2-elf     # <可选项> 编译器前缀
+
+## solution component
+# hw_info:
+#   cpu_id: cpu0
+#   board_name: pangu_cpu0
+#   cpu_name: CK805EF
+#   vendor_name: csky
+#   chip_name: 'chip_pangu'
+#   ld_script: configs/gcc_eflash.ld.S
+
+## board component
+# hw_info:
+#   chip_name: chip_pangu
+#   ld_script: configs/gcc_eflash.ld
+
+## chip component (单处理器)
+# hw_info:
+#   cpu_name: CK804EF
+#   ld_script: configs/gcc_eflash.ld
+#   toolchain_prefix: csky-abiv2-elf
+#   flash_program: bin/flash_program.elf
+
+## chip component(多处理器)
+# hw_info:
+#   cpu0:
+#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
+#     ld_script: configs/gcc_eflash.ld     # <必选项> 连接时使用 LD 脚本
+#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
+#     toolchain_prefix: csky-abiv2-elf     # <可选项> 编译器前缀
+#   cpu1:
+#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
+#     ld_script: configs/gcc_eflash.ld     # <必选项> 连接时使用 LD 脚本
+#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
+#     toolchain_prefix: csky-abiv2-elf     # <可选项> 编译器前缀
+'''
+
         if text:
-            text = 'hw_info:\n' + text
+            if hw_key in ['board', 'chip', 'solution']:
+                text =text_head + hw_key + ':\n' + text
+            else:
+                text = 'hw_info:\n' + text
 
         return text
 
@@ -135,7 +215,7 @@ class build_config:
 
 
 class Package(object):
-    def __init__(self, filename):
+    def __init__(self, filename=None):
         self.filename = filename
         self.name = ''
         self.description = ''
@@ -156,7 +236,8 @@ class Package(object):
         self.export = []
         self.hw_info = HardwareInfo({})
 
-        self.load(filename)
+        if filename:
+            self.load(filename)
 
     def load(self, filename):
         conf = yaml_load(filename)
@@ -195,17 +276,17 @@ class Package(object):
             if not v:
                 continue
             if k in ['build_config']:
-                print(k)
+                put_string(k)
                 for kk, vv, in v.__dict__.items():
-                    print("  ", kk, ":", vv)
+                    put_string("  ", kk, ":", vv)
             elif k in ['defconfig']:
-                print(k)
+                put_string(k)
                 for k1, v1 in v.items():
-                    print("  ", k1, ":", v1)
+                    put_string("  ", k1, ":", v1)
             else:
-                print(k, str(v))
+                put_string(k, str(v))
 
-    def save(self):
+    def dumps(self):
         text = '''
 ## 第一部分： 基础信息
 {name}{version}{description}{type}
@@ -216,57 +297,7 @@ class Package(object):
 # depends:                                 # <可选项> 该组件依赖其他的组件，合理的依赖才能保证组件能编译、使用
 #   - minilibc: v7.2.0
 #   - aos: >=v7.2.0
-{depends}
-## 第三部分：板级信息
-# hw_info:                                 # 对 solution、board、chip 三个字段的改进，统称为 hw_info 字段
-#   cpu_id: cpu0
-#   cpu_name: CK804EF
-#   vendor_name: t-head
-#   chip_name: chip_pangu                  # <可选项> 指定芯片组件名,未设置时，使用depends中 chip 第一个组件
-#   board_name: pangu_cpu0                 # <可选项> 指定开发板组件名，未设置时，使用depends中 board 第一个组件
-#   ld_script: configs/gcc_eflash.ld       # <可选项> 连接时使用 LD 脚本，当选把未设置时，使用对应的 board 的 LD 脚本
-#   flash_program: bin/flash_program.elf   # <可选项> 芯片全局烧写程序
-#   cpu0:
-#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
-#     ld_script: configs/gcc_eflash.ld     # <必选项> 连接时使用 LD 脚本
-#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
-#   cpu1:
-#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
-#     ld_script: configs/gcc_eflash.ld     # <可选项> 连接时使用 LD 脚本
-#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
-
-## solution component
-# hw_info:
-#   cpu_id: cpu0
-#   board_name: pangu_cpu0
-#   cpu_name: CK805EF
-#   vendor_name: csky
-#   chip_name: 'chip_pangu'
-#   ld_script: configs/gcc_eflash.ld.S
-
-## board component
-# hw_info:
-#   chip_name: chip_pangu
-#   ld_script: configs/gcc_eflash.ld
-
-## chip component (单处理器)
-# hw_info:
-#   cpu_name: CK804EF
-#   ld_script: configs/gcc_eflash.ld
-#   flash_program: bin/flash_program.elf
-
-## chip component(多处理器)
-# hw_info:
-#   cpu0:
-#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
-#     ld_script: configs/gcc_eflash.ld     # <必选项> 连接时使用 LD 脚本
-#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
-#   cpu1:
-#     cpu_name: CK804EF                    # <必选项> 该芯片使用的 CPU 型号
-#     ld_script: configs/gcc_eflash.ld     # <必选项> 连接时使用 LD 脚本
-#     flash_program: bin/flash_program.elf # <可选项> 该CPU对应烧写程序，未指定时使用全局烧写程序
-#   flash_program: bin/flash_program.elf   # <可选项> 芯片全局烧写程序
-{hw_info}
+{depends}{hw_info}
 ## 第四部分：编译连接信息
 # build_config:                            # <可选项> 编译配置项
 #   include:                               # <可选项> 编译时，影响编译器的-I 参数 ,全局有效
@@ -295,16 +326,11 @@ class Package(object):
 #   CONFIG_PARAM_NOT_CHECK: y
 #   CONFIG_CLI: y
 {def_config}
-## 第六部分：安装信息
-# install:
-#   - dest: include/                       # 安装的目的路径 dest是相当路径，通常是相对于YoC SDK 安装目录
-#     source:                              # 安装源列表
-#      - src/*.h                           # 支持通配符，相对路径
 {install}
-## 第七部分：导出部分
 {export}'''
 
-        text = text.format(
+
+        return text.format(
             name=self.text_out('name', self.name,
                                '# <必选项> 包名称 （符合C语言变量命名规则），长度少于等于64字节'),
             version=self.text_out('version', self.version),
@@ -323,21 +349,26 @@ class Package(object):
             source_file=self.fsource_file_out(),
             install=self.install_out('install', self.install),
             export=self.install_out('export', self.export),
-            hw_info=self.hw_info.out(),
+            hw_info=self.hw_info.out(self.type),
             build_config=self.build_config.out(),
             def_config=self.def_config_out()
         )
-        # print(text)
-        # return
+
+
+    def save(self):
+        text = self.dumps()
         try:
-            with open(self.filename, 'w') as f:
+            with codecs.open(self.filename, 'w') as f:
                 f.write(text)
             return True
         except:
-            print("Generate %s file failed." % self.filename)
+            put_string("Generate %s file failed." % self.filename)
 
     def text_out(self, k, v, desc=''):
         k_len = len(k)
+        if sys.version_info.major == 2:
+            if type(v) == unicode:
+                v = v.encode('utf8')
         if type(v) != str:
             v = str(v)
         v_len = string_len(v)
@@ -407,7 +438,24 @@ class Package(object):
                 for src in ins['source']:
                     text += '      - "%s"\n' % src
 
-        return text
+        notes = {
+        'install': '''## 第六部分：安装信息
+# install:
+#   - dest: include/                       # 安装的目的路径 dest是相当路径，通常是相对于YoC SDK 安装目录
+#     source:                              # 安装源列表
+#      - src/*.h                           # 支持通配符，相对路径
+''',
+            'export': '''## 第七部分：导出部分
+# export:
+#   - dest: "<SOLUTION_PATH>/generated/data" # 安装的目的路径 dest是相当路径
+#     source:                                # 安装源列表
+#       - "bootimgs/boot"
+#       - "bootimgs/tee"
+#       - "bootimgs/mtb"
+#       - "configs/config.yaml"
+'''}
+        if key in notes:
+            return notes[key] + text
 
     def def_config_out(self):
         text = ''
