@@ -1,8 +1,10 @@
+# -*- coding:utf-8 -*-
+#
+
 import SCons.Action
 import SCons.Builder
 import SCons.Util
 
-
 class ToolObjcopyWarning(SCons.Warnings.Warning):
     pass
 
@@ -21,9 +23,7 @@ def _detect(env):
     except KeyError:
         pass
 
-    product = env.WhereIs('product') or env.WhereIs('/usr/local/bin/product')
-    if product:
-        return product
+    return env.WhereIs('product') or env.WhereIs('/usr/local/bin/product')
 
     raise SCons.Errors.StopError(
         ObjcopyCompilerNotFound,
