@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -43,7 +43,7 @@ class Variables:
             if value != None:
                 var = var.replace('<'+key+'>', self.vars[key])
             else:
-                print('not found variable:', var)
+                put_string('not found variable:', var)
 
         return var
 
@@ -68,56 +68,68 @@ class Solution:
         self.CXXFLAGS = []
         self.LINKFLAGS = []
         self.ld_script = ''
+        self.ld_script_component = None
         self.cpu_name = ''
+        self.toolchain_prefix = ''
 
         # find current solution & board component
         for component in self.components:
             if component.path == os.getcwd() and component.type == 'solution':
                 self.solution_component = component
-                self.board_component = self.components.get(
-                    self.solution_component.hw_info.board_name)
-                if self.board_component:
+                if self.solution_component.hw_info.board_name:
+                    self.board_component = self.components.get(
+                        self.solution_component.hw_info.board_name)
+                    if not self.board_component:
+                        put_string("Not found in `%s` components that the current project depends on. " %
+                                   self.solution_component.hw_info.board_name)
+                        exit(-1)
+
                     self.chip_component = self.components.get(
                         self.board_component.hw_info.chip_name)
+                    if not self.chip_component:
+                        put_string("Not found in `%s` components that the current project depends on. " %
+                                   self.board_component.hw_info.chip_name)
+                        exit(-1)
                 break
 
         if not self.solution_component:
-            logger.error(
-                'No define solution component, please set a solution component')
+            put_string('The current directory is not a solution directory.')
             exit(-1)
 
-        if not self.board_component:
+        if (not self.board_component) and (not self.solution_component.hw_info.cpu_name) and (not self.solution_component.hw_info.ld_script):
             for name in self.solution_component.depends:
                 component = self.components.get(name)
                 if component:
                     if component.type == 'board':
                         self.board_component = component
+                        self.solution_component.hw_info.board_name = self.board_component.name
+                        self.chip_component = self.components.get(
+                            self.board_component.hw_info.chip_name)
+                        if self.chip_component:
+                            self.solution_component.hw_info.chip_name = self.chip_component.name
+                        else:
+                            put_string("Not found chip component: " +
+                                       self.board_component.hw_info.chip_name)
+                            exit(-1)
                         break
                 else:
-                    print("`%s` not found, please install it: yoc install %s" %
-                          (name, name))
+                    put_string("`%s` not found, please install it: yoc install %s" %
+                               (name, name))
                     exit(-1)
             if not self.board_component:
                 logger.warning(
-                    'No define board component, please set a board component')
-            else:
-                self.solution_component.hw_info.board_name = self.board_component.name
+                    'Not found board component in depends')
 
-        if not self.chip_component:
-            for name in self.solution_component.depends:
-                component = self.components.get(name)
-                if component:
-                    if component.type == 'chip':
-                        self.chip_component = component
-                        break
-                else:
-                    logger.error("Not found component: %s" % name)
-                    exit(-1)
-            if not self.chip_component:
-                logger.warning(
-                    'No define chip component, please set a chip component')
-            else:
-                self.solution_component.hw_info.chip_name = self.chip_component.name
+        if self.board_component:
+            self.board_component.need_build = True
+        if self.chip_component:
+            self.chip_component.need_build = True
+
+        components = ComponentGroup()
+        for component in self.components:
+            if component.need_build:
+                components.add(component)
+        self.components = components
 
         def cpu_set_variable(cpu, arch):
             self.cpu_name = cpu
@@ -129,8 +141,14 @@ class Solution:
             #     self.variables.set('cpu_num', '803ef')
             #     self.variables.set('ARCH', 'CSKY', True)
             else:
-                self.variables.set('cpu_num', cpu_id(cpu))
-                self.variables.set('ARCH', 'CSKY', True)
+                if arch == '' or arch.upper() == 'CSKY':
+                    self.variables.set('cpu_num', cpu_id(cpu))
+                    self.variables.set('ARCH', 'CSKY', True)
+                elif arch == 'arm':
+                    self.variables.set('cpu_num', arm_cpu_id(cpu))
+                    self.variables.set('ARCH', arch, True)
+                else:
+                    logger.error("not support arch:%s yet" % arch)
 
         for component in self.components:
             # defines
@@ -159,9 +177,6 @@ class Solution:
 
         # soution 的hw_info 优先
         solution_hw_info.merge(self.solution_component.hw_info)
-        for k, v in self.solution_component.hw_info.__dict__.items():
-            if v:
-                solution_hw_info.__dict__[k] = v
 
         if solution_hw_info.cpu_name == '':
             logger.error("not found cpu_id, please set cpu_id")
@@ -197,13 +212,11 @@ class Solution:
         self.lib_path = os.path.join(self.yoc_path, 'lib')
         self.libpath.append(self.lib_path)
 
-        # save_yoc_config(self.defines, os.path.join(self.solution_component.path, 'include/yoc_config.h'))
-        # save_csi_config(self.defines, os.path.join(self.solution_component.path, 'include/csi_config.h'))
         self.global_includes.append(os.path.join(
             self.solution_component.path, 'include'))
 
         for component in self.components:
-            if len(component.source_file) > 0 and component.type != 'solution' and component.name not in self.libs:
+            if len(component.source_files) > 0 and component.type != 'solution' and component.name not in self.libs:
                 self.libs.append(component.name)
                 self.depend_libs.append(os.path.join(
                     self.lib_path, 'lib' + component.name + '.a'))
@@ -227,58 +240,72 @@ class Solution:
 
         if self.solution_component.hw_info.ld_script:
             self.ld_script = self.solution_component.hw_info.ld_script
+            self.ld_script_component = self.solution_component
         elif self.board_component and self.board_component.hw_info.ld_script:
             self.ld_script = self.board_component.hw_info.ld_script
+            self.ld_script_component = self.board_component
         elif self.chip_component and self.chip_component.hw_info.ld_script:
             self.ld_script = self.chip_component.hw_info.ld_script
+            self.ld_script_component = self.chip_component
         else:
             logger.error("not found LD script file, please set ld_script")
             exit(-1)
 
+        self.toolchain_prefix = solution_hw_info.toolchain_prefix
+        self.ld_script = self.variables.convert(self.ld_script)
+
     def install(self):
         for component in self.components:
             component.install(self.yoc_path)
-        save_yoc_config(self.defines, os.path.join(
-            self.solution_component.path, 'include/yoc_config.h'))
-        save_csi_config(self.defines, os.path.join(
-            self.solution_component.path, 'include/csi_config.h'))
 
-    def show(self):
-        print("[component]")
-        for c in self.components:
-            c.show(4)
+    def show(self, show_component=True, show_libs=True, show_include=True, show_define=True, show_veriable=True, show_flags=True):
+        if show_component:
+            put_string("[component]")
+            for c in self.components:
+                c.show(4)
 
-        if self.solution_component:
-            print("solution: " + self.solution_component.name)
-        if self.board_component:
-            print("board   : " + self.board_component.name)
-        if self.chip_component:
-            print("chip    : " + self.chip_component.name)
+            put_string()
+            if self.solution_component:
+                put_string("    solution component: " +
+                           self.solution_component.name)
+            if self.board_component:
+                put_string("    board component   : " +
+                           self.board_component.name)
+            if self.chip_component:
+                put_string("    chip component    : " +
+                           self.chip_component.name)
 
-        print("[libs]")
-        for v in self.libs:
-            print('    ' + v)
+        if show_libs:
+            put_string("[libs]")
+            for v in self.libs:
+                put_string('    ' + v)
 
-        print("[libpath]")
-        for v in self.libpath:
-            print('    ' + v)
+            put_string("[libpath]")
+            for v in self.libpath:
+                put_string('    ' + v)
 
-        print("[global_includes]")
-        for v in self.global_includes:
-            print('    ' + v)
+        if show_include:
+            put_string("[global_includes]")
+            for v in self.global_includes:
+                put_string('    ' + v)
 
-        print("[defines]")
-        for k, v in self.defines.items():
-            print('    ', k, '=', v)
+        if show_define:
+            put_string("[defines]")
+            for k, v in self.defines.items():
+                put_string('    ', k, '=', v)
 
-        print("[variables]")
-        for k, v in self.variables.items():
-            print('    %s = %s' % (k, v))
+        if show_veriable:
+            put_string("[variables]")
+            for k, v in self.variables.items():
+                put_string('    %s = %s' % (k, v))
 
-        print("ASFLAGS  :" + ' '.join(self.ASFLAGS))
-        print("CCFLAGS  :" + ' '.join(self.CCFLAGS))
-        print("CXXFLAGS :" + ' '.join(self.CXXFLAGS))
-        print("LINKFLAGS:" + ' '.join(self.LINKFLAGS))
+        if show_flags:
+            put_string("ASFLAGS   :" + ' '.join(self.ASFLAGS))
+            put_string("CCFLAGS   :" + ' '.join(self.CCFLAGS))
+            put_string("CXXFLAGS  :" + ' '.join(self.CXXFLAGS))
+            put_string("LINKFLAGS :" + ' '.join(self.LINKFLAGS))
+            put_string('ld_script : ' + self.ld_script)
+            put_string('toolchain : ' + self.toolchain_prefix)
 
 
 def cpu_id(s):
@@ -289,3 +316,12 @@ def cpu_id(s):
             ids += c
         elif ids:
             return ids
+
+
+def arm_cpu_id(s):
+    if s.startswith('arm'):
+        return s[3:]
+    else:
+        arr = s.split('-')
+        if len(arr) > 1:
+            return arr[1]
