@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -15,7 +15,7 @@ class Branch(Command):
 %prog [<compnent> ...]
 """
     helpDescription = """
-Initialize yoc workspace in the current directory.
+View current component branches
 """
     # def _Options(self, p):
     #     p.add_option('-a', '--all',
@@ -28,15 +28,15 @@ Initialize yoc workspace in the current directory.
             for component in yoc.components:
                 git = GitRepo(component.path, component.repo_url)
                 branches = git.GetRemoteBranches()
-                print(component.name + ':')
+                put_string(component.name + ':')
                 for b in branches:
-                    print('  ', b.name)
+                    put_string('  ', b.name)
         else:
             for arg in args:
                 component = yoc.components.get(arg)
                 if component:
                     git = GitRepo(component.path, component.repo_url)
                     branches = git.GetRemoteBranches()
-                    print(component.name + ':')
+                    put_string(component.name + ':')
                     for b in branches:
-                        print('  ', b.name)
+                        put_string('  ', b.name)
