@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -11,12 +11,12 @@ import shutil
 
 class Export(Command):
     common = True
-    helpSummary = "component export"
+    helpSummary = "Export component to dest directory"
     helpUsage = """
 %prog <component> [<component> <component> ...] <dest path>
 """
     helpDescription = """
-component export.
+Export component to dest directory
 """
 
     def Execute(self, opt, args):
@@ -27,14 +27,19 @@ component export.
         componentList = args[:-1]
         destPath = args[-1]
 
+        count = 0
+
         for component in yoc.components:
             if component.name in componentList:
                 component.load_package()
                 depend = yoc.check_depend(component)
                 for d in depend:
                     self.component_export(d, destPath)
+                    count += 1
                 self.component_export(component, destPath)
-        yoc.conf.save(os.path.join(destPath, '.yoc'))
+                count += 1
+        if count:
+            yoc.conf.save(os.path.join(destPath, '.yoc'))
 
 
     def component_export(self, component, path):
@@ -46,7 +51,7 @@ component export.
             p = 'solutions'
         if p:
             dest = os.path.join(path, p, component.name)
-            print("export `%s` to %s" % (component.name, dest))
+            put_string("export `%s` to %s" % (component.name, dest))
             try:
                 shutil.copytree(component.path, dest)
             except:
