@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -23,8 +23,7 @@ Format component package.yaml file
 
         count = len(args)
         for component in yoc.components:
-            if component.name in args or count == 0:
-                print("formating package: %s" % component.name)
+            if count == 0 or component.name in args:
+                put_string("formating package: %s" % component.name)
                 component.load_package()
                 component.pack.save()
-
