@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -12,7 +12,7 @@ class Install(Command):
     common = True
     helpSummary = "Install component into project environment"
     helpUsage = """
-%prog [<component>...]
+%prog [option] [<component>...]
 """
     helpDescription = """
 Install component into project environment
