@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -9,9 +9,9 @@ from yoctools import *
 
 class Upload(Command):
     common = True
-    helpSummary = "upload compoent to OCC"
+    helpSummary = "Upload compoent to OCC"
     helpUsage = """
-%prog [<component>...]
+%prog [option] [<component>...]
 """
     helpDescription = """
 upload compoent to OCC
@@ -21,8 +21,16 @@ upload compoent to OCC
         p.add_option('-g', '--git',
                      dest='update_git', action='store_true',
                      help='upload code to git repo')
+        p.add_option('-o', '--occ',
+                     dest='update_occ', action='store_true',
+                     help='upload code to OCC')
+
 
     def Execute(self, opt, args):
+        if not (opt.update_git or opt.update_occ):
+            self.Usage()
+            return
+
         yoc = YoC()
         count = len(args)
         if opt.update_git:
@@ -35,9 +43,9 @@ upload compoent to OCC
                     if ssh_url:
                         component.upload()
 
-        else:
+        if opt.update_occ:
             if count == 0:
                 yoc.uploadall()
             else:
                 for name in args:
-                    yoc.upload(name)
+                    yoc.upload(name)
\ No newline at end of file
