@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -12,23 +12,23 @@ class Variable(Command):
     common = True
     helpSummary = "Display variables for the current project"
     helpUsage = """
-%prog [<variable>...]
+%prog [option] [<variable>...]
 """
     helpDescription = """
 Display variables for the current project
 """
-    # def _Options(self, p):
-    #     p.add_option('-a', '--all',
-    #                  dest='show_all', action='store_true',
-    #                  help='show the complete list of commands')
+    def _Options(self, p):
+        p.add_option('-b', '--board',
+                     dest='board_name', action='store', type='str', default=None,
+                     help='show all source code file')
 
     def Execute(self, opt, args):
         yoc = YoC()
-        solution = yoc.getSolution()
+        solution = yoc.getSolution(opt.board_name)
         if len(args) == 0:
             for k, v in solution.variables.items():
-                print("%-10s = %s" % (k, v))
+                put_string("%-10s = %s" % (k, v))
         else:
             for arg in args:
                 var = solution.variables.get(arg)
-                print(var)
+                put_string(var)
