@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2015-2020 Alibaba Group Holding Limited
+# Copyright (C) 2019-2020 Alibaba Group Holding Limited
 
 
 from __future__ import print_function
@@ -28,32 +28,76 @@ except:
 try:
     import yaml
 except:
-    print("\n\nNot found pyyaml, please install: \nsudo pip install pyyaml")
+    put_string("\n\nNot found pyyaml, please install: \nsudo pip install pyyaml")
     sys.exit(0)
 
 
 def string_len(text):
     L = 0
     R = ''
+    if sys.version_info.major == 2:
+        if type(text) == str:
+            text = text.decode('utf8')
     for c in text:
-        if c >= '\u4E00' and c <= '\u9FA5':
+        if ord(c) >= 0x4E00 and ord(c) <= 0x9FA5:
             L += 2
         else:
             L += 1
     return L
 
 
+def put_string(*args, **keys):
+    for a in args:
+        if sys.version_info.major == 2:
+            if type(a) == unicode:
+                a = a.encode('utf8')
+        if 'key' in keys:
+            key = keys['key']
+            color_print(a, key=key, end=' ')
+        else:
+            print(a, end=' ')
+    print()
+
+
+def color_print(text, key=[], end='\n'):
+    idx = {}
+    for k in key:
+        index = 0
+        while True:
+            s = text.find(k, index)
+            if s >= 0:
+                e = s + len(k)
+                need_del = []
+                for a, b in idx.items():
+                    if max(s, a) <= min(e, b):
+                        s = min(s, a)
+                        e = max(e, b)
+                        need_del.append(a)
+                for v in need_del:
+                    del idx[v]
+                idx[s] = e
+                index = e
+            else:
+                break
+    s = 0
+    for v in list(sorted(idx)):
+        e = v
+        print(text[s: e], end='')
+        print('\033[1;31m' + text[v: idx[v]] + '\033[0m', end='')
+        s = idx[v]
+    print(text[s: len(text)], end=end)
+
+
 def yaml_load(filename):
     try:
-        with codecs.open(filename) as fh:
+        with codecs.open(filename, 'r', encoding='UTF-8') as fh:
             text = fh.read()
             return yaml.safe_load(text)
     except Exception as e:
-        print(e)
+        put_string(e)
         exit(-1)
 
 
-
 def home_path(path=''):
     return os.path.join(os.path.expanduser('~'), path)
 
@@ -82,9 +126,8 @@ def http_request(method, url, data=None, headers={}):
     connection.request(method=method, url=conn.path + '?' +
                        conn.query, body=data, headers=headers)
     response = connection.getresponse()
-    return response.getcode(), response.read(), response.headers
-    # data = response.read()
-    # return json.load(response)
+    return response.status, response.read(), response.msg
+
 
 
 def http_get(url, path):
@@ -101,7 +144,7 @@ def http_get(url, path):
     filename = os.path.join(path, os.path.basename(conn.path))
 
     try:
-        with open(filename, 'wb') as f:
+        with codecs.open(filename, 'wb', encoding='UTF-8') as f:
             f.write(response.read())
     except:
         pass
@@ -122,7 +165,7 @@ def wget(url, out_file):
         f = sys.stdout
         pervent = recv_size / totalsize
         percent_str = "%.2f%%" % (pervent * 100)
-        n = round(pervent * 50)
+        n = int(pervent * 50)
         s = ('#' * n).ljust(50, '-')
         f.write(percent_str.ljust(8, ' ') + '[' + s + ']' + speed_str)
         f.flush()
@@ -144,7 +187,6 @@ def wget(url, out_file):
     return urlretrieve(url, out_file, barProcess)
 
 
-
 # make_archive(base_name, format, root_dir=None, base_dir=None, verbose=0,dry_run=0, owner=None, group=None, logger=None)
 def dfs_get_zip_file(input_path, result):
     files = os.listdir(input_path)
@@ -193,7 +235,7 @@ def write_file(text, filename):
     contents = None
 
     try:
-        with open(filename, 'r') as f:
+        with codecs.open(filename, 'r', encoding='UTF-8') as f:
             contents = f.read()
     except:
         pass
@@ -207,10 +249,10 @@ def write_file(text, filename):
         except:
             pass
 
-        with open(filename, 'w') as f:
+        with codecs.open(filename, 'w', encoding='UTF-8') as f:
             f.write(text)
     except:
-        print("Generate %s file failed." % filename)
+        put_string("Generate %s file failed." % filename)
 
 
 def genSConcript(path):
@@ -277,7 +319,7 @@ def save_yoc_config(defines, filename):
     contents = ""
 
     try:
-        with open(filename, 'r') as f:
+        with codecs.open(filename, 'r', encoding='UTF-8') as f:
             contents = f.read()
     except:
         pass
